# Copyright 2020, Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

package Google::Ads::GoogleAds::V12::Enums::ConversionLagBucketEnum;

use strict;
use warnings;

use Const::Exporter enums => [
  UNSPECIFIED                 => "UNSPECIFIED",
  UNKNOWN                     => "UNKNOWN",
  LESS_THAN_ONE_DAY           => "LESS_THAN_ONE_DAY",
  ONE_TO_TWO_DAYS             => "ONE_TO_TWO_DAYS",
  TWO_TO_THREE_DAYS           => "TWO_TO_THREE_DAYS",
  THREE_TO_FOUR_DAYS          => "THREE_TO_FOUR_DAYS",
  FOUR_TO_FIVE_DAYS           => "FOUR_TO_FIVE_DAYS",
  FIVE_TO_SIX_DAYS            => "FIVE_TO_SIX_DAYS",
  SIX_TO_SEVEN_DAYS           => "SIX_TO_SEVEN_DAYS",
  SEVEN_TO_EIGHT_DAYS         => "SEVEN_TO_EIGHT_DAYS",
  EIGHT_TO_NINE_DAYS          => "EIGHT_TO_NINE_DAYS",
  NINE_TO_TEN_DAYS            => "NINE_TO_TEN_DAYS",
  TEN_TO_ELEVEN_DAYS          => "TEN_TO_ELEVEN_DAYS",
  ELEVEN_TO_TWELVE_DAYS       => "ELEVEN_TO_TWELVE_DAYS",
  TWELVE_TO_THIRTEEN_DAYS     => "TWELVE_TO_THIRTEEN_DAYS",
  THIRTEEN_TO_FOURTEEN_DAYS   => "THIRTEEN_TO_FOURTEEN_DAYS",
  FOURTEEN_TO_TWENTY_ONE_DAYS => "FOURTEEN_TO_TWENTY_ONE_DAYS",
  TWENTY_ONE_TO_THIRTY_DAYS   => "TWENTY_ONE_TO_THIRTY_DAYS",
  THIRTY_TO_FORTY_FIVE_DAYS   => "THIRTY_TO_FORTY_FIVE_DAYS",
  FORTY_FIVE_TO_SIXTY_DAYS    => "FORTY_FIVE_TO_SIXTY_DAYS",
  SIXTY_TO_NINETY_DAYS        => "SIXTY_TO_NINETY_DAYS"
];

1;
